@@ -1,7 +1,7 @@
 /*
  * Hydrogen
  * Copyright(c) 2002-2008 by Alex >Comix< Cominu [comix@users.sourceforge.net]
- * Copyright(c) 2008-2023 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
+ * Copyright(c) 2008-2024 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
  *
  * http://www.hydrogen-music.org
  *
@@ -289,6 +289,11 @@ std::shared_ptr<Instrument> Instrument::load_from( XMLNode* pNode, const QString
 			sInstrumentDrumkitPath = pNode->read_string( "drumkitPath", "",
 														 false, false, bSilent  );
 
+#ifdef H2CORE_HAVE_APPIMAGE
+			sInstrumentDrumkitPath =
+				Filesystem::rerouteDrumkitPath( sInstrumentDrumkitPath );
+#endif
+
 			// Check whether corresponding drumkit exist.
 			// When tweaking or assembling drumkits locally their
 			// absolute paths serve as unique identifiers to keep them
