@@ -1,7 +1,7 @@
 /*
  * Hydrogen
  * Copyright(c) 2002-2008 by Alex >Comix< Cominu [comix@users.sourceforge.net]
- * Copyright(c) 2008-2023 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
+ * Copyright(c) 2008-2024 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
  *
  * http://www.hydrogen-music.org
  *
@@ -46,6 +46,7 @@
 #include <core/Basics/AutomationPath.h>
 #include <core/AutomationPathSerializer.h>
 #include <core/Hydrogen.h>
+#include <core/Helpers/Future.h>
 #include <core/Helpers/Legacy.h>
 #include <core/Sampler/Sampler.h>
 #include <core/SoundLibrary/SoundLibraryDatabase.h>
@@ -238,6 +239,8 @@ std::shared_ptr<Song> Song::loadFrom( XMLNode* pRootNode, const QString& sFilena
 
 	std::shared_ptr<Song> pSong = std::make_shared<Song>( sName, sAuthor, fBpm, fVolume );
 
+	pSong->setIsMuted( pRootNode->read_bool( "isMuted", false, true, false,
+											 bSilent ) );
 	pSong->setMetronomeVolume( pRootNode->read_float( "metronomeVolume", 0.5,
 													  false, false, bSilent ) );
 	pSong->setNotes( pRootNode->read_string( "notes", "...", false, false, bSilent ) );
@@ -369,32 +372,52 @@ std::shared_ptr<Song> Song::loadFrom( XMLNode* pRootNode, const QString& sFilena
 	}
 	pSong->setPanLawKNorm( fPanLawKNorm );
 
-	XMLNode componentListNode = pRootNode->firstChildElement( "componentList" );
-	if ( ( ! componentListNode.isNull()  ) ) {
-		XMLNode componentNode = componentListNode.firstChildElement( "drumkitComponent" );
-		while ( ! componentNode.isNull()  ) {
-			auto pDrumkitComponent = DrumkitComponent::load_from( &componentNode );
-			if ( pDrumkitComponent != nullptr ) {
-				pSong->getComponents()->push_back( pDrumkitComponent );
-			}
-
-			componentNode = componentNode.nextSiblingElement( "drumkitComponent" );
-		}
+	XMLNode drumkitNode = pRootNode->firstChildElement( "drumkit_info" );
+	if ( ! drumkitNode.isNull() ) {
+		// Starting with Hydrogen 1.3.0 a proper drumkit will be stored as part
+		// of a song.
+		auto pDrumkitComponents = Future::loadDrumkitComponentsFromKit( &drumkitNode );
+		pSong->m_pComponents = std::make_shared<std::vector<std::shared_ptr<DrumkitComponent>>>(pDrumkitComponents);
 	}
 	else {
-		auto pDrumkitComponent = std::make_shared<DrumkitComponent>( 0, "Main" );
-		pSong->getComponents()->push_back( pDrumkitComponent );
+		XMLNode componentListNode = pRootNode->firstChildElement( "componentList" );
+		if ( ( ! componentListNode.isNull()  ) ) {
+			XMLNode componentNode = componentListNode.firstChildElement( "drumkitComponent" );
+			while ( ! componentNode.isNull()  ) {
+				auto pDrumkitComponent = DrumkitComponent::load_from( &componentNode );
+				if ( pDrumkitComponent != nullptr ) {
+					pSong->getComponents()->push_back( pDrumkitComponent );
+				}
+
+				componentNode = componentNode.nextSiblingElement( "drumkitComponent" );
+			}
+		}
+		else {
+			auto pDrumkitComponent = std::make_shared<DrumkitComponent>( 0, "Main" );
+			pSong->getComponents()->push_back( pDrumkitComponent );
+		}
 	}
 
 	// Instrument List
-	//
-	// By supplying no drumkit path the individual drumkit meta infos
-	// stored in the 'instrument' nodes will be used.
-	auto pInstrumentList = InstrumentList::load_from( pRootNode,
-													  "", // sDrumkitPath
-													  "", // sDrumkitName
-													  License(), // per-instrument licenses
-													  bSilent );
+	std::shared_ptr<InstrumentList> pInstrumentList;
+	if ( ! drumkitNode.isNull() ) {
+		// >= 1.3.0
+		pInstrumentList = InstrumentList::load_from( &drumkitNode,
+													 "", // sDrumkitPath
+													 "", // sDrumkitName
+													 License(), // per-instrument licenses
+													 bSilent );
+	}
+	else {
+		// By supplying no drumkit path the individual drumkit meta infos
+		// stored in the 'instrument' nodes will be used.
+		pInstrumentList = InstrumentList::load_from( pRootNode,
+													 "", // sDrumkitPath
+													 "", // sDrumkitName
+													 License(), // per-instrument licenses
+													 bSilent );
+	}
+
 	if ( pInstrumentList == nullptr ) {
 		return nullptr;
 	}
@@ -406,6 +429,11 @@ std::shared_ptr<Song> Song::loadFrom( XMLNode* pRootNode, const QString& sFilena
 		pRootNode->read_string( "last_loaded_drumkit", "", true, false, true );
 	QString sLastLoadedDrumkitName = 
 		pRootNode->read_string( "last_loaded_drumkit_name", "", true, false, true );
+
+#ifdef H2CORE_HAVE_APPIMAGE
+	sLastLoadedDrumkitPath =
+		Filesystem::rerouteDrumkitPath( sLastLoadedDrumkitPath );
+#endif
 	
 	if ( sLastLoadedDrumkitPath.isEmpty() ) {
 		// Prior to version 1.2.0 the last loaded drumkit was read
@@ -439,20 +467,19 @@ std::shared_ptr<Song> Song::loadFrom( XMLNode* pRootNode, const QString& sFilena
 	}
 	pSong->setLastLoadedDrumkitPath( sLastLoadedDrumkitPath );
 
+	// Attempt to access the last loaded drumkit to load it into the
+	// SoundLibraryDatabase in case it was a custom one (e.g. loaded
+	// via OSC or from a different system data folder due to a
+	// different install prefix).
+	auto pSoundLibraryDatabase = Hydrogen::get_instance()->getSoundLibraryDatabase();
+	auto pDrumkit = pSoundLibraryDatabase->getDrumkit( sLastLoadedDrumkitPath, true );
+
 	if ( sLastLoadedDrumkitName.isEmpty() ) {
 		// The initial song is loaded after Hydrogen was
 		// bootstrap. So, the SoundLibrary should be present and
 		// filled with all available drumkits.
-		auto pSoundLibraryDatabase = Hydrogen::get_instance()->getSoundLibraryDatabase();
-		if ( pSoundLibraryDatabase != nullptr ) {
-
-			// If it is not already present, we also load the last
-			// loaded drumkit into the database.
-			auto pDrumkit = pSoundLibraryDatabase->getDrumkit(
-				sLastLoadedDrumkitPath, true );
-			if ( pDrumkit != nullptr ) {
-				sLastLoadedDrumkitName = pDrumkit->get_name();
-			}
+		if ( pSoundLibraryDatabase != nullptr && pDrumkit != nullptr ) {
+			sLastLoadedDrumkitName = pDrumkit->get_name();
 		}
 	}
 	pSong->setLastLoadedDrumkitName( sLastLoadedDrumkitName );
@@ -754,6 +781,7 @@ void Song::writeTo( XMLNode* pRootNode, bool bSilent ) {
 	pRootNode->write_string( "version", QString( get_version().c_str() ) );
 	pRootNode->write_float( "bpm", m_fBpm );
 	pRootNode->write_float( "volume", m_fVolume );
+	pRootNode->write_bool( "isMuted", m_bIsMuted );
 	pRootNode->write_float( "metronomeVolume", m_fMetronomeVolume );
 	pRootNode->write_string( "name", m_sName );
 	pRootNode->write_string( "author", m_sAuthor );
