@@ -1,7 +1,7 @@
 /*
  * Hydrogen
  * Copyright(c) 2002-2008 by Alex >Comix< Cominu [comix@users.sourceforge.net]
- * Copyright(c) 2008-2023 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
+ * Copyright(c) 2008-2024 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
  *
  * http://www.hydrogen-music.org
  *
@@ -210,7 +210,7 @@ bool CoreActionController::setStripPanSym( int nStrip, float fValue, bool bSelec
 			pHydrogen->setSelectedInstrumentNumber( nStrip );
 		}
 
-		return sendStripPanSymFeedback( nStrip );
+		return sendStripPanFeedback( nStrip );
 	}
 
 	return false;
@@ -404,33 +404,6 @@ bool CoreActionController::sendStripPanFeedback( int nStrip ) {
 	return false;
 }
 
-bool CoreActionController::sendStripPanSymFeedback( int nStrip ) {
-	auto pInstr = getStrip( nStrip );
-	if ( pInstr != nullptr ) {
-
-#ifdef H2CORE_HAVE_OSC
-		if ( Preferences::get_instance()->getOscFeedbackEnabled() ) {
-			std::shared_ptr<Action> pFeedbackAction =
-				std::make_shared<Action>( "PAN_ABSOLUTE_SYM" );
-		
-			pFeedbackAction->setParameter1( QString("%1").arg( nStrip + 1 ) );
-			pFeedbackAction->setValue( QString("%1")
-									   .arg( pInstr->getPan() ) );
-			OscServer::get_instance()->handleAction( pFeedbackAction );
-		}
-#endif
-	
-		MidiMap* pMidiMap = MidiMap::get_instance();
-		auto ccParamValues = pMidiMap->findCCValuesByActionParam1( QString("PAN_ABSOLUTE_SYM"),
-																   QString("%1").arg( nStrip ) );
-
-		return handleOutgoingControlChanges( ccParamValues,
-											 pInstr->getPan() * 127 );
-	}
-
-	return false;
-}
-
 bool CoreActionController::handleOutgoingControlChanges( std::vector<int> params, int nValue)
 {
 	Preferences *pPref = Preferences::get_instance();
@@ -629,6 +602,9 @@ bool CoreActionController::setSong( std::shared_ptr<Song> pSong, bool bRelinking
 	if ( pHydrogen->getGUIState() != Hydrogen::GUIState::unavailable ) {
 		EventQueue::get_instance()->push_event( EVENT_UPDATE_SONG, 0 );
 	}
+
+	// As we just set a fresh song, we can mark it not modified
+	pHydrogen->setIsModified( false );
 	
 	return true;
 }
@@ -911,8 +887,9 @@ bool CoreActionController::activateSongMode( bool bActivate ) {
 
 	auto pHydrogen = Hydrogen::get_instance();
 	auto pAudioEngine = pHydrogen->getAudioEngine();
+	auto pSong = pHydrogen->getSong();
 
-	if ( pHydrogen->getSong() == nullptr ) {
+	if ( pSong == nullptr ) {
 		ERRORLOG( "no song set" );
 		return false;
 	}
@@ -926,17 +903,16 @@ bool CoreActionController::activateSongMode( bool bActivate ) {
 	pHydrogen->sequencer_stop();
 
 	pAudioEngine->lock( RIGHT_HERE );
-	pAudioEngine->reset( true );
-	
+
 	if ( bActivate && pHydrogen->getMode() != Song::Mode::Song ) {
 		pHydrogen->setMode( Song::Mode::Song );
-	} else if ( ! bActivate && pHydrogen->getMode() != Song::Mode::Pattern ) {
+	}
+	else if ( ! bActivate && pHydrogen->getMode() != Song::Mode::Pattern ) {
 		pHydrogen->setMode( Song::Mode::Pattern );
 	}
+	
+	pAudioEngine->handleSongModeChanged();
 
-	// Ensure the playing patterns are properly updated regardless of
-	// the state of transport before switching song modes.
-	pAudioEngine->updatePlayingPatterns();
 	pAudioEngine->unlock();
 	
 	return true;
@@ -974,6 +950,10 @@ bool CoreActionController::activateLoopMode( bool bActivate ) {
 		}
 		bChange = true;
 	}
+
+	pAudioEngine->lock( RIGHT_HERE );
+	pAudioEngine->handleLoopModeChanged();
+	pAudioEngine->unlock();
 	
 	if ( bChange ) {
 		EventQueue::get_instance()->push_event( EVENT_LOOP_MODE_ACTIVATION,
