@@ -1,7 +1,7 @@
 /*
  * Hydrogen
  * Copyright(c) 2002-2008 by Alex >Comix< Cominu [comix@users.sourceforge.net]
- * Copyright(c) 2008-2023 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
+ * Copyright(c) 2008-2024 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
  *
  * http://www.hydrogen-music.org
  *
@@ -221,6 +221,18 @@ public:
 	QString				m_sMidiDriver;
 	QString				m_sMidiPortName;
 	QString				m_sMidiOutputPortName;
+	/**
+	 * Choice of #m_sMidiPortName and #m_sMidiOutputPortName in case
+	 * no port/device was selected.
+	 *
+	 * Pinning its value to "None" will prevent Hydrogen to connect to
+	 * ports/devices using this exact name but is still done for
+	 * backward compatibility.
+	 */
+	static QString getNullMidiPort() {
+		return "None";
+	}
+	
 	int					m_nMidiChannelFilter;
 	bool				m_bMidiNoteOffIgnore;
 	bool				m_bMidiFixedMapping;
@@ -388,7 +400,7 @@ public:
 	~Preferences();
 
 	/// Load the preferences file
-	void			loadPreferences( bool bGlobal );
+	bool			loadPreferences( bool bGlobal );
 
 	/// Save the preferences file
 	bool			savePreferences();
@@ -529,6 +541,12 @@ public:
 	WindowProperties	getLadspaProperties( unsigned nFX );
 	void			setLadspaProperties( unsigned nFX, const WindowProperties& prop );
 
+	WindowProperties	getPlaylistDialogProperties();
+	void				setPlaylistDialogProperties( const WindowProperties& prop );
+
+	WindowProperties	getDirectorProperties();
+	void				setDirectorProperties( const WindowProperties& prop );
+
 	const std::shared_ptr<ColorTheme>	getColorTheme() const;
 	void			setColorTheme( const std::shared_ptr<ColorTheme> pNewColorTheme );
 
@@ -654,6 +672,8 @@ public:
 
 	const std::shared_ptr<Theme> getTheme() const;
 	void setTheme( const std::shared_ptr<Theme> pTheme );
+
+	bool getLoadingSuccessful() const;
 	
 private:
 	/**
@@ -748,6 +768,8 @@ private:
 	WindowProperties		instrumentRackProperties;
 	WindowProperties		audioEngineInfoProperties;
 	WindowProperties		m_ladspaProperties[MAX_FX];
+	WindowProperties		m_playlistDialogProperties;
+	WindowProperties		m_directorProperties;
 
 	QString					m_sPreferredLanguage;
 
@@ -800,6 +822,8 @@ private:
 
 	WindowProperties readWindowProperties( XMLNode parent, const QString& windowName, WindowProperties defaultProp );
 	void writeWindowProperties( XMLNode parent, const QString& windowName, const WindowProperties& prop );
+
+	bool m_bLoadingSuccessful;
 };
 
 inline QString			Preferences::getLastExportPatternAsDirectory() const {
@@ -1310,6 +1334,20 @@ inline void Preferences::setLadspaProperties( unsigned nFX, const WindowProperti
 	m_ladspaProperties[nFX] = prop;
 }
 
+inline WindowProperties Preferences::getPlaylistDialogProperties() {
+	return m_playlistDialogProperties;
+}
+inline void Preferences::setPlaylistDialogProperties( const WindowProperties& prop ) {
+	m_playlistDialogProperties = prop;
+}
+
+inline WindowProperties Preferences::getDirectorProperties() {
+	return m_directorProperties;
+}
+inline void Preferences::setDirectorProperties( const WindowProperties& prop ) {
+	m_directorProperties = prop;
+}
+
 inline const std::shared_ptr<ColorTheme> Preferences::getColorTheme() const {
 	return m_pTheme->getColorTheme();
 }
@@ -1429,6 +1467,9 @@ inline void Preferences::setTheme( const std::shared_ptr<Theme> pTheme ) {
 inline const std::shared_ptr<Theme> Preferences::getTheme() const {
 	return m_pTheme;
 }
+inline bool Preferences::getLoadingSuccessful() const {
+	return m_bLoadingSuccessful;
+}
 };
 
 #endif
