@@ -1,6 +1,7 @@
 /*
  * Hydrogen
  * Copyright(c) 2002-2008 by Alex >Comix< Cominu [comix@users.sourceforge.net]
+ * Copyright(c) 2008-2024 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
  *
  * http://www.hydrogen-music.org
  *
@@ -15,8 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program. If not, see https://www.gnu.org/licenses
  *
  */
 
@@ -26,13 +26,12 @@
 
 #include "ui_LadspaFXSelector_UI.h"
 
-#include <hydrogen/config.h>
-#include <hydrogen/object.h>
+#include <core/config.h>
+#include <core/Object.h>
 
 #include <QtGui>
-#if QT_VERSION >= 0x050000
-#  include <QtWidgets>
-#endif
+#include <QtWidgets>
+
 #include <string>
 #include <vector>
 
@@ -41,13 +40,14 @@ namespace H2Core {
 	class LadspaFXGroup;
 }
 
-class LadspaFXSelector : public QDialog, public Ui_LadspaFXSelector_UI, public H2Core::Object
+/** \ingroup docGUI*/
+class LadspaFXSelector :  public QDialog, public Ui_LadspaFXSelector_UI,  public H2Core::Object<LadspaFXSelector>
 {
-    H2_OBJECT
+    H2_OBJECT(LadspaFXSelector)
 	Q_OBJECT
 
 	public:
-		LadspaFXSelector(int nLadspaFX);
+		explicit LadspaFXSelector(int nLadspaFX);
 		~LadspaFXSelector();
 
 		QString getSelectedFX();
