@@ -1,6 +1,7 @@
 /*
  * Hydrogen
  * Copyright(c) 2002-2008 by Alex >Comix< Cominu [comix@users.sourceforge.net]
+ * Copyright(c) 2008-2024 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
  *
  * http://www.hydrogen-music.org
  *
@@ -15,8 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program. If not, see https://www.gnu.org/licenses
  *
  */
 
@@ -24,36 +24,33 @@
 #ifndef MIXER_H
 #define MIXER_H
 
-
 #include <QtGui>
-#if QT_VERSION >= 0x050000
-#  include <QtWidgets>
-#endif
+#include <QtWidgets>
 
-#include <hydrogen/object.h>
-#include <hydrogen/globals.h>
+#include <core/Object.h>
+#include <core/Preferences/Preferences.h>
+#include <core/Globals.h>
 #include "../EventListener.h"
 
 class Button;
-class ToggleButton;
 class MixerLine;
 class ComponentMixerLine;
-class FxMixerLine;
 class MasterMixerLine;
 class LadspaFXMixerLine;
 class PixmapWidget;
 
-class Mixer : public QWidget, public EventListener, public H2Core::Object
+/** \ingroup docGUI*/
+class Mixer :  public QWidget, public EventListener,  public H2Core::Object<Mixer>
 {
-	H2_OBJECT
+	H2_OBJECT(Mixer)
 	Q_OBJECT
 	public:
-		Mixer(QWidget* parent);
+		explicit Mixer(QWidget* parent);
 		~Mixer();
 
-		void showEvent ( QShowEvent *ev );
-		void hideEvent ( QHideEvent *ev );
-		void resizeEvent ( QResizeEvent *ev );
+		void showEvent ( QShowEvent *ev ) override;
+		void hideEvent ( QHideEvent *ev ) override;
+		void resizeEvent ( QResizeEvent *ev ) override;
 		void soloClicked(uint nLine);
 		bool isSoloClicked(uint nLine);
 
@@ -64,8 +61,6 @@ class Mixer : public QWidget, public EventListener, public H2Core::Object
 		void noteOffClicked(MixerLine* ref);
 		void muteClicked(MixerLine* ref);
 		void muteClicked(ComponentMixerLine* ref);
-		void unmuteAll( bool findSelectedInstr);
-		void unmuteAll( int selectedInstr);
 		void soloClicked(MixerLine* ref);
 		void soloClicked(ComponentMixerLine* ref);
 		void volumeChanged(MixerLine* ref);
@@ -76,20 +71,23 @@ class Mixer : public QWidget, public EventListener, public H2Core::Object
 		void nameClicked(MixerLine* ref);
 		void nameSelected(MixerLine* ref);
 		void updateMixer();
-		void showFXPanelClicked(Button* ref);
-		void showPeaksBtnClicked(Button* ref);
-		void ladspaActiveBtnClicked( LadspaFXMixerLine* ref );
+		void showFXPanelClicked();
+		void showPeaksBtnClicked();
+		void openMixerSettingsDialog();
+		void ladspaBypassBtnClicked( LadspaFXMixerLine* ref );
 		void ladspaEditBtnClicked( LadspaFXMixerLine *ref );
-		void ladspaVolumeChanged( LadspaFXMixerLine* ref);
-		void closeEvent(QCloseEvent *event);
+		void ladspaVolumeChanged( LadspaFXMixerLine *ref );
+		void closeEvent(QCloseEvent *event) override;
+		void onPreferencesChanged( H2Core::Preferences::Changes changes );
 
 	private:
 		QHBoxLayout *			m_pFaderHBox;
 		LadspaFXMixerLine *		m_pLadspaFXLine[MAX_FX];
 
 		QScrollArea*			m_pFaderScrollArea;
-		ToggleButton *			m_pShowFXPanelBtn;
-		ToggleButton *			m_pShowPeaksBtn;
+		Button *				m_pShowFXPanelBtn;
+		Button *				m_pShowPeaksBtn;
+		Button *				m_pOpenMixerSettingsBtn;
 		MasterMixerLine *		m_pMasterLine;
 
 		QWidget *				m_pFaderPanel;
@@ -100,14 +98,14 @@ class Mixer : public QWidget, public EventListener, public H2Core::Object
 
 		QTimer *				m_pUpdateTimer;
 
-		uint findMixerLineByRef(MixerLine* ref);
-		uint findCompoMixerLineByRef(ComponentMixerLine* ref);
-		MixerLine* createMixerLine( int );
-		ComponentMixerLine* createComponentMixerLine( int );
+		uint					findMixerLineByRef(MixerLine* ref);
+		uint					findCompoMixerLineByRef(ComponentMixerLine* ref);
+		MixerLine*				createMixerLine( int );
+		ComponentMixerLine*		createComponentMixerLine( int );
 
 		// Implements EventListener interface
-		virtual void noteOnEvent( int nInstrument );
-		//~ Implements EventListener interface
+		virtual void noteOnEvent( int nInstrument ) override;
+		// ~ Implements EventListener interface
 
 };
 
