@@ -1,6 +1,7 @@
 /*
  * Hydrogen
  * Copyright(c) 2002-2008 by Alex >Comix< Cominu [comix@users.sourceforge.net]
+ * Copyright(c) 2008-2024 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
  *
  * http://www.hydrogen-music.org
  *
@@ -15,8 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program. If not, see https://www.gnu.org/licenses
  *
  */
 
@@ -25,15 +25,18 @@
 #define PATTERN_EDITOR_INSTRUMENT_LIST_H
 
 
-#include <hydrogen/globals.h>
+#include <core/Globals.h>
 
 #include <QtGui>
-#if QT_VERSION >= 0x050000
-#  include <QtWidgets>
-#endif
+#include <QtWidgets>
 
-#include <hydrogen/object.h>
+#include <core/Object.h>
+#include <core/Preferences/Preferences.h>
 #include "../Widgets/PixmapWidget.h"
+#include "../EventListener.h"
+#include "../Selection.h"
+#include "../Widgets/WidgetWithScalableFont.h"
+#include "../Widgets/WidgetWithHighlightedList.h"
 
 namespace H2Core
 {
@@ -41,21 +44,32 @@ namespace H2Core
 }
 
 class PatternEditorPanel;
-class ToggleButton;
+class Button;
 
+/** \ingroup docGUI*/
 class InstrumentLine : public PixmapWidget
+					 , protected WidgetWithScalableFont<8, 10, 12>
+					 , protected WidgetWithHighlightedList
 {
-    H2_OBJECT
+    H2_OBJECT(InstrumentLine)
 	Q_OBJECT
 
 	public:
-		InstrumentLine(QWidget* pParent);
+		explicit InstrumentLine(QWidget* pParent);
 
+	int getNumber() const;
+	
 		void setName(const QString& sName);
 		void setSelected(bool isSelected);
 		void setNumber(int nIndex);
 		void setMuted(bool isMuted);
 		void setSoloed( bool soloed );
+		void setSamplesMissing( bool bSamplesMissing );
+
+	static constexpr int m_nButtonWidth = 18;
+
+public slots:
+		void onPreferencesChanged( H2Core::Preferences::Changes changes );
 
 	private slots:
 		void functionClearNotes();
@@ -69,51 +83,73 @@ class InstrumentLine : public PixmapWidget
 		void functionFillEveryTwelveNotes();
 		void functionFillEverySixteenNotes();
 		void functionFillNotes( int every );
-		void functionCopyInstrumentPattern();
 		void functionCopyAllInstrumentPatterns();
-		void functionPasteInstrumentPattern();
 		void functionPasteAllInstrumentPatterns();
-		void functionPasteInstrumentPatternExec(int patternID);
+		void functionDeleteNotesAllPatterns();
+		void functionCutNotesAllPatterns();
 
 		void functionRandomizeVelocity();
-		void functionDeleteInstrument();
 		void functionRenameInstrument();
 		void muteClicked();
 		void soloClicked();
+		void sampleWarningClicked();
+
+		void selectInstrumentNotes();
 
 
 	private:
 		QMenu *m_pFunctionPopup;
 		QMenu *m_pFunctionPopupSub;
-		QMenu *m_pCopyPopupSub;
-		QMenu *m_pPastePopupSub;
 		QLabel *m_pNameLbl;
 		bool m_bIsSelected;
 		int m_nInstrumentNumber;	///< The related instrument number
-		ToggleButton *m_pMuteBtn;
-		ToggleButton *m_pSoloBtn;
+		Button *m_pMuteBtn;
+		Button *m_pSoloBtn;
+		Button *m_pSampleWarning;
 
-		virtual void mousePressEvent(QMouseEvent *ev);
+		virtual void mousePressEvent(QMouseEvent *ev) override;
+	virtual void mouseDoubleClickEvent( QMouseEvent* ev ) override;
+	virtual void enterEvent( QEvent *ev );
+	virtual void leaveEvent( QEvent *ev );
+	virtual void paintEvent( QPaintEvent* ev ) override;
 		H2Core::Pattern* getCurrentPattern();
+
+	void updateStyleSheet();
+	void setRowSelection( RowSelection rowSelection );
+
+	/** Whether the cursor entered the boundary of the widget.*/
+	bool m_bEntered;
 };
 
+inline int InstrumentLine::getNumber() const {
+	return m_nInstrumentNumber;
+}
 
-class PatternEditorInstrumentList : public QWidget, public H2Core::Object {
-	H2_OBJECT
+
+/** \ingroup docGUI*/
+class PatternEditorInstrumentList :  public QWidget,
+									 public EventListener,
+									 public H2Core::Object<PatternEditorInstrumentList> {
+	H2_OBJECT(PatternEditorInstrumentList)
 	Q_OBJECT
 
 	public:
 		PatternEditorInstrumentList( QWidget *parent, PatternEditorPanel *pPatternEditorPanel );
 		~PatternEditorInstrumentList();
 
-		virtual void mousePressEvent(QMouseEvent *event);
-		virtual void mouseMoveEvent(QMouseEvent *event);
+		virtual void mousePressEvent(QMouseEvent *event) override;
+		virtual void mouseMoveEvent(QMouseEvent *event) override;
 
 
-		virtual void dragEnterEvent(QDragEnterEvent *event);
-		virtual void dropEvent(QDropEvent *event);
-
+		virtual void dragEnterEvent(QDragEnterEvent *event) override;
+		virtual void dropEvent(QDropEvent *event) override;
 
+	virtual void selectedInstrumentChangedEvent() override;
+	virtual void updateSongEvent( int nEvent ) override;
+	virtual void drumkitLoadedEvent() override;
+	virtual void instrumentParametersChangedEvent( int ) override;
+	
+	void repaintInstrumentLines();
 	public slots:
 		void updateInstrumentLines();
 
@@ -126,11 +162,14 @@ class PatternEditorInstrumentList : public QWidget, public H2Core::Object {
 		uint m_nEditorHeight;
 		InstrumentLine* m_pInstrumentLine[MAX_INSTRUMENTS];
 		QTimer *m_pUpdateTimer;
+		DragScroller *m_pDragScroller;
 
 		QPoint __drag_start_position;
 
 		InstrumentLine* createInstrumentLine();
 
+private:
+	void drawFocus( QPainter& painter );
 };
 
 
