@@ -1,6 +1,7 @@
 /*
  * Hydrogen
  * Copyright(c) 2002-2008 by Alex >Comix< Cominu [comix@users.sourceforge.net]
+ * Copyright(c) 2008-2024 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
  *
  * http://www.hydrogen-music.org
  *
@@ -15,23 +16,19 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program. If not, see https://www.gnu.org/licenses
  *
  */
 
 #include "PatternPropertiesDialog.h"
-#include "Skin.h"
 #include "HydrogenApp.h"
 #include "UndoActions.h"
 
-#include <hydrogen/hydrogen.h>
-#include <hydrogen/basics/pattern.h>
-#include <hydrogen/basics/pattern_list.h>
-#include <hydrogen/Preferences.h>
+#include <core/Hydrogen.h>
+#include <core/Basics/Pattern.h>
+#include <core/Basics/PatternList.h>
+#include <core/Preferences/Preferences.h>
 
-
-using namespace std;
 using namespace H2Core;
 
 PatternPropertiesDialog::PatternPropertiesDialog(QWidget* parent, Pattern *pattern, int nselectedPattern, bool savepattern)
@@ -98,6 +95,10 @@ void PatternPropertiesDialog::on_okBtn_clicked()
 	QString pattCategory = categoryComboBox->currentText();
 	QString pattInfo = patternDescTxt->toPlainText();
 
+	// Ensure the pattern name is unique
+	PatternList *pPatternList = Hydrogen::get_instance()->getSong()->getPatternList();
+	pattName = pPatternList->find_unused_pattern_name(pattName, pattern);
+
 	Preferences *pPref = H2Core::Preferences::get_instance();
 	std::list<QString>::const_iterator cur_testpatternCategories;
 
@@ -117,8 +118,7 @@ void PatternPropertiesDialog::on_okBtn_clicked()
 		pattern->set_name( pattName );
 		pattern->set_info( pattInfo );
 		pattern->set_category( pattCategory );
-	}else
-	{
+	} else if ( pattern->get_name() != pattName || pattern->get_info() != pattInfo || pattern->get_category() != pattCategory) {
 		SE_modifyPatternPropertiesAction *action = new SE_modifyPatternPropertiesAction(  pattern->get_name() , pattern->get_info(), pattern->get_category(),
 												  pattName, pattInfo, pattCategory, __nselectedPattern );
 		HydrogenApp::get_instance()->m_pUndoStack->push( action );
@@ -128,50 +128,10 @@ void PatternPropertiesDialog::on_okBtn_clicked()
 
 void PatternPropertiesDialog::defaultNameCheck( QString pattName, bool savepattern)
 {
-	if ( savepattern && !nameCheck(pattName) )
-	{
-		defaultNameCheck( tr( "%1#2").arg(pattName), savepattern );
-	}
-	else
-	{
-		patternNameTxt->setText( tr( "%1").arg(pattName) );
-	}
-}
-
-
-bool PatternPropertiesDialog::nameCheck( QString pattName )
-{
-	if (pattName == "") {
-		return false;
-	}
-	PatternList *patternList = Hydrogen::get_instance()->getSong()->get_pattern_list();
-	
-	for (uint i = 0; i < patternList->size(); i++) {
-		if ( patternList->get(i)->get_name() == pattName) {
-			return false;
-		}
-	}
-	return true;
-}
-
-
-void PatternPropertiesDialog::on_categoryComboBox_editTextChanged()
-{
-	if ( categoryComboBox->currentText() == pattern->get_category() ) {
-		okBtn->setEnabled( false );
-	}
-	else {
-		okBtn->setEnabled(true);
-	}
-}
-
-
-void PatternPropertiesDialog::on_patternNameTxt_textChanged()
-{
-	if ( nameCheck( patternNameTxt->text() ) ) {
-		okBtn->setEnabled( true );
-	}
-	else {
-		okBtn->setEnabled( false );
+	PatternList *pPatternList = Hydrogen::get_instance()->getSong()->getPatternList();
+	if ( savepattern && !pPatternList->check_name(pattName, pattern) ) {
+		pattName = pPatternList->find_unused_pattern_name(pattName, pattern);
 	}
+
+	patternNameTxt->setText(pattName);
 }
