@@ -1,7 +1,7 @@
 /*
  * Hydrogen
  * Copyright(c) 2002-2008 by Alex >Comix< Cominu [comix@users.sourceforge.net]
- * Copyright(c) 2008-2023 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
+ * Copyright(c) 2008-2024 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
  *
  * http://www.hydrogen-music.org
  *
@@ -58,6 +58,7 @@ int bcDisplaystatus = 0;
 PlayerControl::PlayerControl(QWidget *parent)
  : QLabel(parent)
  , m_midiActivityTimeout( 125 )
+ , m_bLCDBPMSpinboxIsArmed( true )
 {
 
 	m_pHydrogen = Hydrogen::get_instance();
@@ -548,7 +549,9 @@ void PlayerControl::updatePlayerControl()
 
 	if ( ! m_pLCDBPMSpinbox->hasFocus() &&
 		 ! m_pLCDBPMSpinbox->getIsHovered() ) {
+		m_bLCDBPMSpinboxIsArmed = false;
 		m_pLCDBPMSpinbox->setValue( m_pHydrogen->getAudioEngine()->getTransportPosition()->getBpm() );
+		m_bLCDBPMSpinboxIsArmed = true;
 	}
 
 	//beatcounter
@@ -767,7 +770,8 @@ void PlayerControl::activateSongMode( bool bActivate ) {
 
 void PlayerControl::bpmChanged( double fNewBpmValue ) {
 	auto pAudioEngine = m_pHydrogen->getAudioEngine();
-	if ( m_pLCDBPMSpinbox->getIsActive() ) {
+	if ( m_pLCDBPMSpinbox->getIsActive() &&
+		 m_bLCDBPMSpinboxIsArmed ) {
 		// Store it's value in the .h2song file.
 		m_pHydrogen->getSong()->setBpm( static_cast<float>( fNewBpmValue ) );
 		// Use tempo in the next process cycle of the audio engine.
@@ -1058,6 +1062,11 @@ void PlayerControl::updateBeatCounterToolTip() {
 
 void PlayerControl::tempoChangedEvent( int nValue )
 {
+	// When updating the tempo of the BPM spin box it is crucial to
+	// indicated that this was done due to a batch event and not due
+	// to user input.
+	m_bLCDBPMSpinboxIsArmed = false;
+
 	// Also update value if the BPM widget is disabled
 	bool bIsReadOnly = m_pLCDBPMSpinbox->isReadOnly();
 
@@ -1072,11 +1081,14 @@ void PlayerControl::tempoChangedEvent( int nValue )
 	 * of the song.
 	 */
 	m_pLCDBPMSpinbox->setValue( m_pHydrogen->getAudioEngine()->getTransportPosition()->getBpm() );
-	
+
 	if ( ! bIsReadOnly ) {
 		m_pLCDBPMSpinbox->setReadOnly( false );
 	}
 
+	// Re-enabling core Bpm alteration using BPM spinbox
+	m_bLCDBPMSpinboxIsArmed = true;
+
 	if ( nValue == -1 ) {
 		// Value was changed via API commands and not by the
 		// AudioEngine.
