@@ -1,7 +1,7 @@
 /*
  * Hydrogen
  * Copyright(c) 2002-2008 by Alex >Comix< Cominu [comix@users.sourceforge.net]
- * Copyright(c) 2008-2023 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
+ * Copyright(c) 2008-2024 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
  *
  * http://www.hydrogen-music.org
  *
@@ -35,6 +35,7 @@
 #include <QTreeWidgetItemIterator>
 #include "../Widgets/MidiTable.h"
 
+#include <core/EventQueue.h>
 #include <core/MidiMap.h>
 #include <core/Hydrogen.h>
 #include <core/IO/MidiInput.h>
@@ -135,7 +136,8 @@ PreferencesDialog::PreferencesDialog(QWidget* parent)
 	: QDialog( parent )
 	, m_pCurrentColor( nullptr )
 	, m_nCurrentId( 0 )
-	, m_changes( H2Core::Preferences::Changes::None ) {
+	, m_changes( H2Core::Preferences::Changes::None )
+	, m_bMidiTableChanged( false ) {
 	
 	m_pCurrentTheme = std::make_shared<H2Core::Theme>( H2Core::Preferences::get_instance()->getTheme() );
 	m_pPreviousTheme = std::make_shared<H2Core::Theme>( H2Core::Preferences::get_instance()->getTheme() );
@@ -223,24 +225,9 @@ PreferencesDialog::PreferencesDialog(QWidget* parent)
 	driverComboBox->setSize( audioTabWidgetSizeTop );
 	driverComboBox->clear();
 	driverComboBox->addItem( "Auto" );
-#ifdef H2CORE_HAVE_JACK
-	driverComboBox->addItem( "JACK" );
-#endif
-#ifdef H2CORE_HAVE_ALSA
-	driverComboBox->addItem( "ALSA" );
-#endif
-#ifdef H2CORE_HAVE_OSS
-	driverComboBox->addItem( "OSS" );
-#endif
-#ifdef H2CORE_HAVE_PORTAUDIO
-	driverComboBox->addItem( "PortAudio" );
-#endif
-#ifdef H2CORE_HAVE_COREAUDIO
-	driverComboBox->addItem( "CoreAudio" );
-#endif
-#ifdef H2CORE_HAVE_PULSEAUDIO
-	driverComboBox->addItem( "PulseAudio" );
-#endif
+	for ( const QString& ssDriver : pHydrogen->getAudioEngine()->getSupportedAudioDrivers() ) {
+		driverComboBox->addItem( ssDriver );
+	}
 
 	if( driverComboBox->findText(pPref->m_sAudioDriver) > -1){
 		driverComboBox->setCurrentIndex(driverComboBox->findText(pPref->m_sAudioDriver));
@@ -381,7 +368,7 @@ PreferencesDialog::PreferencesDialog(QWidget* parent)
 	// MIDI tab - list midi input ports
 	midiPortComboBox->clear();
 	midiPortComboBox->addItem( pCommonStrings->getPreferencesNone() );
-	if ( pHydrogen->getMidiInput() ) {
+	if ( pHydrogen->getMidiInput() != nullptr ) {
 		std::vector<QString> midiOutList = pHydrogen->getMidiInput()->getOutputPortList();
 
 		if ( midiOutList.size() != 0 ) {
@@ -402,7 +389,7 @@ PreferencesDialog::PreferencesDialog(QWidget* parent)
 	midiOutportComboBox->setSize( midiTabWidgetSize );
 	midiOutportComboBox->clear();
 	midiOutportComboBox->addItem( pCommonStrings->getPreferencesNone() );
-	if ( pHydrogen->getMidiOutput() ) {
+	if ( pHydrogen->getMidiOutput() != nullptr ) {
 		std::vector<QString> midiOutList = pHydrogen->getMidiOutput()->getInputPortList();
 
 		if ( midiOutList.size() != 0 ) {
@@ -438,6 +425,7 @@ PreferencesDialog::PreferencesDialog(QWidget* parent)
 				 m_changes =
 					 static_cast<H2Core::Preferences::Changes>(
 							m_changes | H2Core::Preferences::Changes::MidiTab );
+				 m_bMidiTableChanged = true;
 			 });
 
 	//////
@@ -500,8 +488,11 @@ PreferencesDialog::PreferencesDialog(QWidget* parent)
 	uiScalingPolicyComboBox->setSize( appearanceTabWidgetSize );
 	iconColorComboBox->setSize( appearanceTabWidgetSize );
 	coloringMethodAuxSpinBox->setSize( appearanceTabWidgetSize );
-	
-	connect( uiLayoutComboBox, SIGNAL( currentIndexChanged(int) ), this, SLOT( onUILayoutChanged(int) ) );
+
+	connect( styleComboBox, SIGNAL( activated(int) ), this,
+			 SLOT( styleComboBoxActivated(int) ) );
+	connect( uiLayoutComboBox, SIGNAL( currentIndexChanged(int) ), this,
+			 SLOT( onUILayoutChanged(int) ) );
 	connect( uiScalingPolicyComboBox, SIGNAL( currentIndexChanged(int) ), this,
 			 SLOT( uiScalingPolicyComboBoxCurrentIndexChanged(int) ) );
 	connect( mixerFalloffComboBox, SIGNAL( currentIndexChanged(int) ), this,
@@ -836,11 +827,7 @@ void PreferencesDialog::on_okBtn_clicked()
 											pCommonStrings->getButtonOk(),
 											pCommonStrings->getButtonCancel(),
 											nullptr, 1 );
-		if ( res == 0 ) {
-			QApplication::setOverrideCursor( Qt::WaitCursor );
-			pHydrogen->restartDrivers();
-			QApplication::restoreOverrideCursor();			
-		} else {
+		if ( res != 0 ) {
 			// Don't save the Preferences and don't close the PreferencesDialog
 			return;
 		}
@@ -910,6 +897,9 @@ void PreferencesDialog::on_okBtn_clicked()
 	mM->reset_instance();
 
 	midiTable->saveMidiTable();
+	if ( m_bMidiTableChanged ) {
+		H2Core::EventQueue::get_instance()->push_event( H2Core::EVENT_MIDI_MAP_CHANGED, 0 );
+	}
 
 	if ( m_pMidiDriverComboBox->currentText() == "ALSA" &&
 		 pPref->m_sMidiDriver != "ALSA" ) {
@@ -956,7 +946,7 @@ void PreferencesDialog::on_okBtn_clicked()
 
 	QString sNewMidiPortName = midiPortComboBox->currentText();
 	if ( midiPortComboBox->currentIndex() == 0 ) {
-		sNewMidiPortName = "None";
+		sNewMidiPortName = Preferences::getNullMidiPort();
 	}
 	if ( pPref->m_sMidiPortName != sNewMidiPortName ) {
 		pPref->m_sMidiPortName = sNewMidiPortName;
@@ -966,7 +956,7 @@ void PreferencesDialog::on_okBtn_clicked()
 	
 	QString sNewMidiOutputPortName = midiOutportComboBox->currentText();
 	if ( midiOutportComboBox->currentIndex() == 0 ) {
-		sNewMidiOutputPortName = "None";
+		sNewMidiOutputPortName = Preferences::getNullMidiPort();
 	}
 	if ( pPref->m_sMidiOutputPortName != sNewMidiOutputPortName ) {
 		pPref->m_sMidiOutputPortName = sNewMidiOutputPortName;
@@ -1105,6 +1095,14 @@ void PreferencesDialog::on_okBtn_clicked()
 
 	pPref->setTheme( m_pCurrentTheme );
 
+	if ( m_bNeedDriverRestart ) {
+		// Restart audio and MIDI drivers now that we updated all
+		// values in Preferences.
+		QApplication::setOverrideCursor( Qt::WaitCursor );
+		pHydrogen->restartDrivers();
+		QApplication::restoreOverrideCursor();
+	}
+
 	pH2App->changePreferences( m_changes );
 	
 	pPref->savePreferences();
@@ -1263,7 +1261,7 @@ void PreferencesDialog::updateDriverInfoLabel() {
 			.append( "</font></b>" );
 #else
 		auto pAlsaDriver =
-			dynamic_cast<H2Core::AlsaAudioDriver*>(Hydrogen::get_instance()->getAudioOutput());
+			dynamic_cast<H2Core::AlsaAudioDriver*>(pAudioDriver);
 		if ( pAlsaDriver != nullptr ) {
 			sInfo.append( "<br>" ).append( tr( "Currently connected to device: " ) )
 				.append( "<b>" ).append( pAlsaDriver->m_sAlsaAudioDevice )
@@ -1461,7 +1459,17 @@ void PreferencesDialog::setDriverInfoPortAudio() {
 	latencyTargetLabel->show();
 	latencyTargetSpinBox->show();
 	latencyValueLabel->show();
-	latencyValueLabel->setText( QString("Current: %1 frames").arg( H2Core::Hydrogen::get_instance()->getAudioOutput()->getLatency() ) );
+
+	const auto pAudioDriver = H2Core::Hydrogen::get_instance()->getAudioOutput();
+	int nLatency;
+	if ( pAudioDriver == nullptr ) {
+		ERRORLOG( "AudioDriver is not ready!" );
+		return;
+	}
+	else {
+		nLatency = pAudioDriver->getLatency();
+	}
+	latencyValueLabel->setText( QString("Current: %1 frames").arg( nLatency ) );
 
 	bufferSizeSpinBox->setToolTip( "" );
 	sampleRateComboBox->setToolTip( "" );
@@ -1760,19 +1768,24 @@ void PreferencesDialog::midiOutportComboBoxActivated( int index )
 void PreferencesDialog::styleComboBoxActivated( int index )
 {
 	UNUSED( index );
-	QApplication *pQApp = (HydrogenApp::get_instance())->getMainForm()->m_pQApp;
-	QString sStyle = styleComboBox->currentText();
-	pQApp->setStyle( sStyle );
-
-	Preferences *pPref = Preferences::get_instance();
-	pPref->setQTStyle( sStyle );
-	m_pCurrentTheme->getInterfaceTheme()->m_sQTStyle = sStyle;
-
-	m_changes =
-		static_cast<H2Core::Preferences::Changes>(
-			m_changes | H2Core::Preferences::Changes::AppearanceTab );
 	
-	HydrogenApp::get_instance()->changePreferences( H2Core::Preferences::Changes::AppearanceTab );
+	QString sStyle = styleComboBox->currentText();
+	if ( sStyle != m_pCurrentTheme->getInterfaceTheme()->m_sQTStyle ) {
+
+		// Instant visual feedback.
+		QApplication *pQApp = (HydrogenApp::get_instance())->getMainForm()->m_pQApp;
+		pQApp->setStyle( sStyle );
+		Preferences *pPref = Preferences::get_instance();
+		pPref->setQTStyle( sStyle );
+
+		m_pCurrentTheme->getInterfaceTheme()->m_sQTStyle = sStyle;
+
+		m_changes =
+			static_cast<H2Core::Preferences::Changes>(
+				m_changes | H2Core::Preferences::Changes::AppearanceTab );
+	
+		HydrogenApp::get_instance()->changePreferences( H2Core::Preferences::Changes::AppearanceTab );
+	}
 }
 
 
