@@ -1,7 +1,7 @@
 /*
  * Hydrogen
  * Copyright(c) 2002-2008 by Alex >Comix< Cominu [comix@users.sourceforge.net]
- * Copyright(c) 2008-2023 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
+ * Copyright(c) 2008-2024 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
  *
  * http://www.hydrogen-music.org
  *
@@ -23,6 +23,8 @@
 #include <QtGui>
 #include <QtWidgets>
 #include <QLibraryInfo>
+#include <QProcess>
+#include <QSslSocket>
 
 #include <core/config.h>
 #include <core/Version.h>
@@ -218,7 +220,7 @@ int main(int argc, char *argv[])
 		QCommandLineParser parser;
 		
 		QString aboutText = QString( "\nHydrogen " ) + QString::fromStdString( H2Core::get_version() )  + QString( " [" ) + QString::fromStdString( __DATE__ ) + QString( "]  [http://www.hydrogen-music.org]" ) +
-		QString( "\nCopyright 2002-2008 Alessandro Cominu\nCopyright 2008-2023 The hydrogen development team" ) +
+		QString( "\nCopyright 2002-2008 Alessandro Cominu\nCopyright 2008-2024 The hydrogen development team" ) +
 		QString( "\nHydrogen comes with ABSOLUTELY NO WARRANTY\nThis is free software, and you are welcome to redistribute it under certain conditions. See the file COPYING for details.\n" );
 		
 		parser.setApplicationDescription( aboutText );
@@ -452,6 +454,54 @@ int main(int argc, char *argv[])
 		Skin::setPalette( pQApp );
 		setApplicationIcon(pQApp);
 
+		if ( ! pPref->getLoadingSuccessful() ) {
+
+			QMessageBox::critical( nullptr, "Hydrogen",
+				QString( QT_TRANSLATE_NOOP( "Startup",															   "No [hydrogen.conf] file found. Hydrogen was not installed properly. Aborting..." ) ) );
+			
+			// Neither the Preferences on system level nor the ones at
+			// user level could be loaded successfully. Hydrogen was
+			// most probably not installed properly. Abort.
+			delete pQApp;
+			delete pPref;
+
+			delete MidiMap::get_instance();
+
+			___ERRORLOG( "No preferences file found. Aborting..." );
+			delete H2Core::Logger::get_instance();
+
+			exit( 1 );
+		}
+
+#ifdef H2CORE_HAVE_APPIMAGE
+		// Within an AppImage we provide our own version of Qt. But we
+		// also have to ensure to provide an OpenSSL shared object
+		// compatible with that particular version. As shipping
+		// `libssl.so` and `libcrypto.so` is discouraged and
+		// blacklisted by `linuxdeploy-qt`, we manually put them in
+		// the application folder. This way Qt tries to load the
+		// system's libraries first and only falls back to the ones we
+		// provide in case it couldn't find it.
+		//
+		// With the following logs we can determine if a fallback did
+		// happen.
+		
+		QString sCurrentDir = QDir::currentPath();
+		QDir::setCurrent( QCoreApplication::applicationDirPath() );
+		QProcess process;
+		process.start( "openssl", QStringList( "version" ));
+		process.waitForFinished(-1);
+		QString sStdout = process.readAllStandardOutput();
+		___INFOLOG( QString( "current: %1, application: %2" ).arg( sCurrentDir )
+					.arg( QCoreApplication::applicationDirPath() ) );
+		___INFOLOG( QString( "OpenSSL supported: [%1], version OS: [%2], Qt runtime: [%3], Qt build: [%4]" )
+					.arg( QSslSocket::supportsSsl() )
+					.arg( sStdout.trimmed() )
+					.arg( QSslSocket::sslLibraryVersionString() )
+					.arg( QSslSocket::sslLibraryBuildVersionString() ) );
+		QDir::setCurrent( sCurrentDir );
+#endif
+
 		SplashScreen *pSplash = new SplashScreen();
 
 #ifdef H2CORE_HAVE_OSC
